/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.workArea.impl;

import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.workArea.WorkArea;
import com.yiring.app.domain.workArea.WorkAreaRepository;
import com.yiring.app.excel.workArea.WorkAreaExportExcel;
import com.yiring.app.param.workArea.WorkAreaParam;
import com.yiring.app.param.workArea.WorkAreaQueryParam;
import com.yiring.app.service.workArea.WorkAreaService;
import com.yiring.app.vo.workArea.WorkAreaVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tzl
 * @version 1.0
 * @description:
 * @date 2022/5/21 9:52
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class WorkAreaServiceImpl implements WorkAreaService {

    @Resource
    WorkAreaRepository workAreaRepository;

    @Override
    public Result<String> saveWorkArea(WorkAreaParam workAreaParam) {
        //工区名称不能重复
        if (hasWorkAreaName(workAreaParam.getWorkAreaName())) {
            return Result.no(Status.BAD_REQUEST, "工区名称已存在");
        }
        WorkArea workArea = new WorkArea();
        BeanUtils.copyProperties(workAreaParam, workArea);
        workAreaRepository.saveAndFlush(workArea);
        return Result.ok();
    }

    @Override
    public Result<String> updateWorkArea(WorkAreaParam workAreaParam, IdParam idParam) {
        Optional<WorkArea> byId = workAreaRepository.findById(idParam.getId());
        if (byId.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        WorkArea workArea = byId.get();
        //修改工区名称时判断
        if (!workArea.getWorkAreaName().equals(workAreaParam.getWorkAreaName())) {
            if (hasWorkAreaName(workAreaParam.getWorkAreaName())) {
                return Result.no(Status.BAD_REQUEST, "工区名称已存在");
            }
        }
        BeanUtils.copyProperties(workAreaParam, workArea);
        workAreaRepository.saveAndFlush(workArea);
        return Result.ok();
    }

    @Override
    public Result<String> deleteWorkArea(IdParam idParam) {
        Optional<WorkArea> byId = workAreaRepository.findById(idParam.getId());
        if (byId.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        WorkArea workArea = byId.get();
        workAreaRepository.delete(workArea);
        return Result.ok();
    }

    @Override
    public Result<PageVo<WorkAreaVo>> pageWorkArea(WorkAreaQueryParam workAreaQueryParam) {
        PageVo<WorkAreaVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(BasicEntity.Fields.updateTime));
        if (Objects.nonNull(workAreaQueryParam.getPageNo()) && Objects.nonNull(workAreaQueryParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(workAreaQueryParam.getPageNo() - 1, workAreaQueryParam.getPageSize());
            Page<WorkArea> all = workAreaRepository.findAll(condition(workAreaQueryParam), pageable);
            List<WorkAreaVo> data = all
                .get()
                .map(workArea -> {
                    WorkAreaVo vo = new WorkAreaVo();
                    BeanUtils.copyProperties(workArea, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<WorkArea> all = workAreaRepository.findAll(condition(workAreaQueryParam), sort);
            List<WorkAreaVo> data = all
                .stream()
                .map(workArea -> {
                    WorkAreaVo vo = new WorkAreaVo();
                    BeanUtils.copyProperties(workArea, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public void export(WorkAreaQueryParam workAreaQueryParam, HttpServletResponse httpServletResponse) {
        List<WorkArea> workAreas = workAreaRepository.findAll(condition(workAreaQueryParam));
        List<WorkAreaExportExcel> workAreaExportExcels = workAreas
            .stream()
            .map(workArea -> {
                WorkAreaExportExcel workAreaExportExcel = new WorkAreaExportExcel();
                BeanUtils.copyProperties(workArea, workAreaExportExcel);
                return workAreaExportExcel;
            })
            .toList();

        try (
            DefaultStreamExcelBuilder<WorkAreaExportExcel> defaultStreamExcelBuilder = DefaultStreamExcelBuilder
                .of(WorkAreaExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .widths(8)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            defaultStreamExcelBuilder.append(workAreaExportExcels);

            String fileName = URLEncoder.encode("工区信息.xlsx", StandardCharsets.UTF_8);
            httpServletResponse.setContentType("application/octet-stream");
            httpServletResponse.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            httpServletResponse.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = httpServletResponse.getOutputStream();

            Workbook workbook = defaultStreamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出工区信息失败: " + e.getMessage());
        }
    }

    public Specification<WorkArea> condition(WorkAreaQueryParam workAreaQueryParam) {
        return (root, cq, cb) -> {
            List<Predicate> list = new ArrayList<>();
            if (StrUtil.isNotBlank(workAreaQueryParam.getWorkAreaName())) {
                list.add(
                    cb.like(root.get(WorkArea.Fields.workAreaName), "%" + workAreaQueryParam.getWorkAreaName() + "%")
                );
            }
            Order order = cb.desc(root.get(BasicEntity.Fields.updateTime));
            Predicate[] predicates = new Predicate[list.size()];
            cq.where(list.toArray(predicates));
            cq.orderBy(order);
            return cb.and(list.toArray(predicates));
        };
    }

    boolean hasWorkAreaName(String wrokAreaName) {
        WorkArea workArea = WorkArea.builder().workAreaName(wrokAreaName).build();
        return workAreaRepository.count(Example.of(workArea)) > 0;
    }
}
