/* (C) 2022 YiRing, Inc. */
package com.yiring.app.util;

import java.io.*;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.LinkedList;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

public class ZipUtils {

    /**
     * 解压文件
     * @param file		压缩文件
     * @param targetDir	解压文件输出的目录
     * @throws IOException IOException
     */
    public static void unPacket(Path file, Path targetDir) throws IOException {
        if (!Files.exists(targetDir)) {
            Files.createDirectories(targetDir);
        }
        // 创建zip对象
        try (ZipFile zipFile = new ZipFile(file.toFile())) {
            // 读取zip流
            try (ZipInputStream zipInputStream = new ZipInputStream(Files.newInputStream(file))) {
                ZipEntry zipEntry;
                // 遍历每一个zip项
                while ((zipEntry = zipInputStream.getNextEntry()) != null) {
                    // 获取zip项目名称
                    String entryName = zipEntry.getName();
                    // 构建绝对路径
                    Path entryFile = targetDir.resolve(entryName);
                    if (zipEntry.isDirectory()) { // 文件夹
                        if (!Files.isDirectory(entryFile)) {
                            Files.createDirectories(entryFile);
                        }
                    } else { // 文件
                        // 读取zip项数据流
                        try (InputStream zipEntryInputStream = zipFile.getInputStream(zipEntry)) {
                            try (
                                OutputStream fileOutputStream = Files.newOutputStream(
                                    entryFile,
                                    StandardOpenOption.CREATE_NEW
                                )
                            ) {
                                byte[] buffer = new byte[4096];
                                int length;
                                while ((length = zipEntryInputStream.read(buffer)) != -1) {
                                    fileOutputStream.write(buffer, 0, length);
                                }
                                fileOutputStream.flush();
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 压缩指定的文件
     * @param files Path[]
     * @param zipFile Path
     * @throws IOException IOException
     */
    public static void packet(Path[] files, Path zipFile) throws IOException {
        OutputStream outputStream = Files.newOutputStream(zipFile, StandardOpenOption.CREATE_NEW);
        try (outputStream; ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {
            for (Path file : files) {
                if (Files.isDirectory(file)) {
                    continue;
                }
                try (InputStream inputStream = Files.newInputStream(file)) {
                    // 创建一个压缩项，指定名称
                    ZipEntry zipEntry = new ZipEntry(file.getFileName().toString());
                    // 添加到压缩流
                    zipOutputStream.putNextEntry(zipEntry);
                    // 写入数据
                    int len;
                    byte[] buffer = new byte[1024 * 10];
                    while ((len = inputStream.read(buffer)) > 0) {
                        zipOutputStream.write(buffer, 0, len);
                    }
                    zipOutputStream.flush();
                }
            }
            // 完成所有压缩项的添加
            zipOutputStream.closeEntry();
        }
    }

    /**
     * 压缩指定的文件
     * @param files	Map<String, File>
     * @param outputStream OutputStream
     */
    public static void packet(Map<String, File> files, OutputStream outputStream) {
        try (outputStream; ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {
            for (Map.Entry<String, File> file : files.entrySet()) {
                String fileName = file.getKey();
                File fileValue = file.getValue();
                try (InputStream inputStream = new BufferedInputStream(new FileInputStream(fileValue))) {
                    // 创建一个压缩项，指定名称
                    ZipEntry zipEntry = new ZipEntry(fileName);
                    // 添加到压缩流
                    zipOutputStream.putNextEntry(zipEntry);
                    // 写入数据
                    int len;
                    byte[] buffer = new byte[1024 * 10];
                    while ((len = inputStream.read(buffer)) > 0) {
                        zipOutputStream.write(buffer, 0, len);
                    }
                    zipOutputStream.flush();
                }
            }
            // 完成所有压缩项的添加
            zipOutputStream.closeEntry();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 压缩指定的目录
     * @param folder Path
     * @param zipFile Path
     * @throws IOException IOException
     */
    public static void packet(Path folder, Path zipFile) throws IOException {
        if (!Files.isDirectory(folder)) {
            throw new IllegalArgumentException(folder + " 不是合法的文件夹");
        }
        OutputStream outputStream = Files.newOutputStream(zipFile, StandardOpenOption.CREATE_NEW);

        LinkedList<String> path = new LinkedList<>();

        try (outputStream; ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {
            Files.walkFileTree(
                folder,
                new FileVisitor<>() {
                    @Override
                    public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) {
                        if (!dir.equals(folder)) {
                            // 开始遍历目录
                            String folder = dir.getFileName().toString();
                            path.addLast(folder);
                            // 写入目录
                            ZipEntry zipEntry = new ZipEntry(path.stream().collect(Collectors.joining("/", "", "/")));
                            try {
                                zipOutputStream.putNextEntry(zipEntry);
                                zipOutputStream.flush();
                            } catch (IOException e) {
                                throw new RuntimeException(e);
                            }
                        }
                        return FileVisitResult.CONTINUE;
                    }

                    @Override
                    public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) {
                        // 开始遍历文件
                        try (InputStream inputStream = Files.newInputStream(file)) {
                            // 创建一个压缩项，指定名称
                            String fileName = path.size() > 0
                                ? path.stream().collect(Collectors.joining("/", "", "")) +
                                "/" +
                                file.getFileName().toString()
                                : file.getFileName().toString();

                            ZipEntry zipEntry = new ZipEntry(fileName);

                            // 添加到压缩流
                            zipOutputStream.putNextEntry(zipEntry);
                            // 写入数据
                            int len;
                            byte[] buffer = new byte[1024 * 10];
                            while ((len = inputStream.read(buffer)) > 0) {
                                zipOutputStream.write(buffer, 0, len);
                            }

                            zipOutputStream.flush();
                        } catch (IOException e) {
                            throw new RuntimeException(e);
                        }
                        return FileVisitResult.CONTINUE;
                    }

                    @Override
                    public FileVisitResult visitFileFailed(Path file, IOException exc) {
                        return FileVisitResult.CONTINUE;
                    }

                    @Override
                    public FileVisitResult postVisitDirectory(Path dir, IOException exc) {
                        // 结束遍历目录
                        if (!path.isEmpty()) {
                            path.removeLast();
                        }
                        return FileVisitResult.CONTINUE;
                    }
                }
            );
            zipOutputStream.closeEntry();
        }
    }
}
